/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#if !defined(HARK_BASE_BASEEXCEPTION_H_INCLUDED)
#define HARK_BASE_BASEEXCEPTION_H_INCLUDED 1

#include "common.h"

#include <iostream>
#include <string>
#include <vector>

namespace HARK
{
    class BaseException
    {
    public:
	virtual ~BaseException();

	virtual void print(std::ostream &out = std::cerr) const = 0;
	virtual void freeze();
	virtual BaseException *add(BaseException *e);
    };

    class GenericCastException : public BaseException
    {
    public:
	virtual void print(std::ostream &out = std::cerr) const = 0;
    };

    class GeneralException :public BaseException
    {
    public:
	GeneralException(std::string _message, std::string _file, int _line);
	virtual void print(std::ostream &out = std::cerr) const;

    protected:
	std::string message;
	std::string file;
	int         line;
    };

    class ExceptionStack : public BaseException
    {
    public:
	ExceptionStack();
	virtual ~ExceptionStack();

	BaseException *add(BaseException *e);
	virtual void print(std::ostream &out = std::cerr) const;
	virtual void freeze();

    private:
	std::vector<BaseException *> stack;

    };
}

#endif

/* end of file */
