/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#if !defined(HARK_BASE_BUFFEREDNODE_H_INCLUDED)
#define HARK_BASE_BUFFEREDNODE_H_INCLUDED 1

#include "common.h"

#include "Node.h"
#include "Buffer.h"

namespace HARK
{
    class OutputCacheInfo
    {
    public:
	OutputCacheInfo();

	RCPtr<Buffer> buffer;
	int lookAhead;
	int lookBack;
    };

    class InputCacheInfo
    {
    public:
	InputCacheInfo();

	int lookAhead;
	int lookBack;
    };

    class BufferedNode : public Node
    {
    protected:
	int processCount;

    public:
	std::vector<OutputCacheInfo> outputs;
	std::vector<InputCacheInfo>  inputsCache;

    protected:
	bool inOrder;
	
    public:
	BufferedNode(const std::string &nodeName, const ParameterSet &params);

	virtual ~BufferedNode();

	virtual ObjectRef getOutput(int output_id, int count);

	virtual void calculate(int output_id, int count, Buffer &buf) = 0;

	virtual void initialize();

	virtual int addOutput (const std::string &outputName);

	virtual int addInput (const std::string &inputName);

	virtual void initializeBuffers();

    protected:
	BufferedNode();

    };

#ifdef ENABLE_SPEED_HACKS

#define NO_ORDER_NODE_SPEEDUP(nodeClass)			\
    ObjectRef getOutput(int output_id, int count)               \
    {                                                           \
	Buffer &outBuffer = *(outputs[output_id].buffer);	\
	if (!outBuffer.isValid(count))				\
	    nodeClass::calculate (output_id, count, outBuffer);	\
	return outBuffer.get(count);}

#define IN_ORDER_NODE_SPEEDUP(nodeClass)			\
    ObjectRef getOutput(int output_id, int count)               \
    {                                                           \
	Buffer &outBuffer = *(outputs[output_id].buffer);	\
	for (int i=processCount+1;i<=count;i++)			\
	    nodeClass::calculate(output_id, i, outBuffer);	\
	if (count > processCount)				\
	    processCount = count;				\
	return outBuffer.get(count);				\
    }


#else

#define NO_ORDER_NODE_SPEEDUP(nodeClass)				\
    ObjectRef getOutput(int output_id, int count)			\
    {									\
	try {								\
	    Buffer &outBuffer = *(outputs[output_id].buffer);		\
	    if (!outBuffer.isValid(count))				\
		nodeClass::calculate (output_id, count, outBuffer);	\
	    return outBuffer.get(count);				\
	} catch (BaseException *e)					\
	    {								\
		throw e->add(new NodeException (this, "Exception caught in BufferedNode::getOutput", __FILE__, __LINE__)); \
	    }								\
    }

#define IN_ORDER_NODE_SPEEDUP(nodeClass)				\
    ObjectRef getOutput(int output_id, int count)			\
    {									\
	try {								\
	    Buffer &outBuffer = *(outputs[output_id].buffer);		\
	    for (int i=processCount+1;i<=count;i++)			\
		nodeClass::calculate(output_id, i, outBuffer);		\
	    if (count > processCount)					\
		processCount = count;					\
	    return outBuffer.get(count);				\
	} catch (BaseException *e)					\
	    {								\
		throw e->add(new NodeException (this, "Exception caught in BufferedNode::getOutput", __FILE__, __LINE__)); \
	    }								\
    }

#endif

}

#endif

/* end of file */
