/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#if !defined(HARK_BASE_CONVERSION_H_INCLUDED)
#define HARK_BASE_CONVERSION_H_INCLUDED 1

#include "common.h"

#include "ObjectRef.h"
#include "typemap.h"

namespace HARK
{
    using ConversionFunc = ObjectRef (*)(ObjectRef);

    class ConversionDispatch
    {
    public:
	using key_type   = std::pair<TypeInfo, TypeInfo>;
	using value_type = ConversionFunc;
	using table_type = std::map<key_type, value_type>;

    public:
	static ConversionDispatch &GetInstance();

	ObjectRef call(const std::type_info *tx, const std::type_info *ty, ObjectRef x);

    private:
	ConversionDispatch();

	template<typename X, typename Y, template<typename, typename> typename F>
	void AddConversionFunc();

    private:
	static ConversionDispatch theInstance;
	table_type dispatchTable;
	
    };

    class Conversion
    {
    public:
	template <class T>
	static ObjectRef convertTo(ObjectRef x)
	{
	    if(x.isNil()){
		return nilObject;
	    }

	    return ConversionDispatch::GetInstance().call(&typeid(*x), &typeid(T), x);
	}
    };

    /*
class Conversion
{
    static TypeMap<TypeMap<conv_func> > &conv_table();
public:
    template<class T, class U>
    static int addConvFunction(conv_func func)
    {
        conv_table()[&typeid(T)][&typeid(U)] = func;
        return 0;
    }

    template <class T>
    static ObjectRef convertTo(ObjectRef x)
    {
        TypeMap<TypeMap<conv_func> >::iterator it = conv_table().find(&typeid(*x));
        if (it != conv_table().end()) {
            TypeMap<conv_func>::iterator it2 = it->second.find(&typeid(T));
            if (it2 != it->second.end()) {
                return it2->second(x);
            } else {
                std::cerr << "Cannot cast this to type requested\nThis needs to throw an exception\n";
                return nilObject;
            }
        } else {
            std::cerr << "Cannot cast\nThis needs to throw an exception\n";
            return nilObject;
        }
    }
};

template<class T, class U>
class dummy_conv_table_init_class
{
    static int dummy_var;
};

#define REGISTER_CONVERSION(from, to, func)			\
    template<>							\
    int dummy_conv_table_init_class<from,to >::dummy_var =	\
        Conversion::addConvFunction<from,to >(func);

#define REGISTER_CONVERSION_TEMPLATE(from, to, func)		\
    template<>							\
    int dummy_conv_table_init_class<from,to >::dummy_var =	\
        Conversion::addConvFunction<from,to >(func<from,to >);
}
    */

}
    
#endif

/* end of file */
