/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#if !defined(HARK_BASE_TYPES_H_INCLUDED)
#define HARK_BASE_TYPES_H_INCLUDED 1

#include <iostream>
#include <typeinfo>

namespace HARK
{
    class Object;

    template<typename T> class RCPtr;
    using ObjectRef = RCPtr<Object>;

    struct TypeInfo
    {
	TypeInfo(const std::type_info *_typeinfo) :
	    typeinfo(_typeinfo)
	{
	}
	
	TypeInfo(const TypeInfo &src) :
	    typeinfo(src.typeinfo)
	{
	}
	
	bool operator == (const TypeInfo &rhs) const
	{
//	    std::cout << "TypeInfo: compare-equal: "
//		      << "lhs=" << this->typeinfo->name() << " (" << (const void *)this->typeinfo << ", " << this->typeinfo->hash_code() << ")"
//		      << ", rhs=" << rhs.typeinfo->name() << " (" << (const void *)rhs.typeinfo << ", " << rhs.typeinfo->hash_code() << ")"
//		      << ", result=" << (*(this->typeinfo) == *(rhs.typeinfo)) << std::endl;
	    return *(this->typeinfo) == *(rhs.typeinfo);
	}

	bool operator < (const TypeInfo &rhs) const
	{
//	    std::cout << "TypeInfo: compare-less: "
//		      << "lhs=" << this->typeinfo->name() << " (" << (const void *)this->typeinfo << ", " << this->typeinfo->hash_code() << ")"
//		      << ", rhs=" << rhs.typeinfo->name() << " (" << (const void *)rhs.typeinfo << ", " << rhs.typeinfo->hash_code() << ")"
//		      << ", result=" << (this->hash() < rhs.hash()) << std::endl;
	    return this->hash() < rhs.hash();
	}

	const char *name() const
	{
	    return this->typeinfo->name();
	}

	operator const void * () const
	{
	    return this->typeinfo;
	}

	size_t hash() const
	{
	    return this->typeinfo->hash_code();
	}

	const std::type_info *typeinfo;
    };

    struct TypeInfoPairHash
    {
	size_t operator () (const std::pair<TypeInfo, TypeInfo> &pair) const
	{
	    return pair.first.hash() + pair.second.hash();
	}
    };
    
    inline std::ostream &operator << (std::ostream &os, const TypeInfo &t)
    {
	os << "TypeInfo:[name=" << t.typeinfo->name() << ", typeinfo=" << (const void *)t.typeinfo << ", this=" << (const void *)&t << "]";
	return os;
    }

}

#endif

/* end of file */
