/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#include "IF.h"

namespace HARK
{
    DECLARE_NODE(IF);
    /*Node
     *
     * @name IF
     * @category Logic
     * @description Takes a branch or another depending on a condition (Bool value).
     *
     * @input_name COND
     * @input_description The condition for the if statement
     * @input_type bool
     *
     * @input_name THEN
     * @input_description What to do if the condition is true
     *
     * @input_name ELSE
     * @input_description What to do if the condition is false
     *
     * @output_name OUTPUT
     * @output_description The object from THEN or ELSE depending on COND
     *
     * @parameter_name PULL_ANYWAY
     * @parameter_type bool
     * @parameter_description If true, the IF statement pulls also on the branch not taken
     *
     END*/

    IF::IF(const std::string &nodeName, const ParameterSet &params) :
	Node(nodeName, params)
    {
	this->_inputID  = this->addInput("COND");
	this->_thenID   = this->addInput("THEN");
	this->_elseID   = this->addInput("ELSE");
	this->_outputID = this->addOutput("OUTPUT");
	if(parameters.exist("PULL_ANYWAY")) {
	    this->_pullAnyway = dereference_cast<bool> (parameters.get("PULL_ANYWAY"));
	} else {
	    this->_pullAnyway = false;
	}
    }

    ObjectRef
    IF::getOutput(int output_id, int count)
    {
	if(dereference_cast<bool> (this->getInput(this->_inputID, count))) {
	    if(this->_pullAnyway) {
		this->getInput(this->_elseID, count);
	    }
	    return this->getInput(this->_thenID, count);
	} else {
	    if(this->_pullAnyway) {
		this->getInput(this->_thenID,count);
	    }
	    return this->getInput(this->_elseID,count);
	}
    }

}

/* end of file */
