/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#include "OR.h"

namespace HARK
{
    DECLARE_NODE(OR);
    /*Node
     *
     * @name OR
     * @category Logic
     * @description Logical OR between two inputs
     *
     * @input_name INPUT1
     * @input_type bool
     * @input_description First boolean input
     *
     * @input_name INPUT2
     * @input_type bool
     * @input_description Second boolean input
     *
     * @output_name OUTPUT
     * @output_type bool
     * @output_description Boolean output
     *
     * @parameter_name PULL_ANYWAY
     * @parameter_type bool
     * @parameter_description Pull on INPUT2 even if INPUT1 is false
     * @parameter_value false
     *
     END*/

    OR::OR(const std::string &nodeName, const ParameterSet &params) :
	BufferedNode(nodeName, params)
    {
	this->_outputID = this->addOutput("OUTPUT");
	if (parameters.exist("PULL_ANYWAY")) {
	    this->_pullAnyway = dereference_cast<bool>(parameters.get("PULL_ANYWAY"));
	} else {
	    this->_pullAnyway = false;
	}
    }

    int
    OR::translateInput(const std::string &inputName)
    {
	for (unsigned int i = 0; i < this->inputs.size(); i++) {
	    if (this->inputs[i].name == inputName) {
		return i;
	    }
	}
	return this->addInput(inputName);
    }

    void
    OR::calculate(int output_id, int count, Buffer &out)
    {
        bool outValue = false;

        for (int j = 0; j < this->inputs.size(); j++) {
            ObjectRef inputValue = this->getInput(j, count);
            if (dereference_cast<bool> (inputValue) && !outValue) {
                outValue = true;
            }
            if (!this->_pullAnyway && outValue) {
                break;
            }
        }
        if (outValue) {
            out[count] = TrueObject;
        } else {
            out[count] = FalseObject;
        }
    }
}

/* end of file */
