/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#include "ParameterSet.h"

namespace HARK
{
    bool
    ParameterSet::exist(const std::string &param) const
    {
	if (this->find(param) != this->end()) {
	    (const_cast <ParameterSet *> (this))->find(param)->second.second = true;
	    return true;
	}
	return false;
    }

    ObjectRef
    ParameterSet::get(const std::string &param) const
    {
	if(this->find(param) == this->end()) {
	    throw new ParameterException("Missing Parameter", param,*this);

	}else{
	    (const_cast <ParameterSet *> (this))->find(param)->second.second = true;
	    return this->find(param)->second.first;
	}
    }

    ObjectRef
    ParameterSet::getDefault(const std::string &param, ObjectRef value)
    {
	if(this->find(param) == this->end()) {
	    return value;
	} else {
	    (const_cast <ParameterSet *> (this))->find(param)->second.second = true;
	    return operator[](param).first;
	}
    }

    void
    ParameterSet::defaultParam(const std::string &param, ObjectRef value)
    {
	if(this->find(param) == this->end()) {
	    (operator[](param)) = std::pair<ObjectRef, bool>(value, false);
	}
    }

    void
    ParameterSet::add(const std::string &param, ObjectRef value)
    {
	(operator[](param)) = std::pair<ObjectRef, bool> (value,false);
    }

    void
    ParameterSet::print(std::ostream &out) const
    {
	for(ParameterSet::const_iterator it = this->begin(); it != this->end(); ++it){
	    out << it->first << " -> " << it->second.first << std::endl;
	}
    }

    void
    ParameterSet::checkUnused() const
    {
	for(ParameterSet::const_iterator it = this->begin(); it != this->end(); ++it) {
	    if(!it->second.second) {
		throw new ParameterException("Unused (unknown) parameter", it->first,*this);
	    }
	}
    }
}

/* end of file */
