/*
 * HARK/hark-base
 *
 * Copyright (C) 2008-2018
 *
 */

#include "Smaller.h"

#include "operators.h"

namespace HARK
{
    DECLARE_NODE(Smaller);
    /*Node
     *
     * @name Smaller
     * @category Logic
     * @description Verifies if INPUT1 > INPUT2
     *
     * @input_name INPUT1
     * @input_description The first operand
     *
     * @input_name INPUT2
     * @input_description The second operand
     *
     * @output_name OUTPUT
     * @output_description bool value
     * @output_type bool
     *
     END*/

    Smaller::Smaller(const std::string &nodeName, const ParameterSet &params) :
        BufferedNode(nodeName, params)
    {
        this->_input1ID = this->addInput("INPUT1");
        this->_input2ID = this->addInput("INPUT2");
        this->_outputID = this->addOutput("OUTPUT");
    }

    void
    Smaller::calculate(int output_id, int count, Buffer &out)
    {
        ObjectRef input1Value = this->getInput(this->_input1ID, count);
        ObjectRef input2Value = this->getInput(this->_input2ID, count);
        out[count] = (input1Value < input2Value);
    }
}

/* end of file */
