#!/usr/bin/python3 -u
#
# Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
# All rights reserved.
# HARK was developed by researchers in Okuno Laboratory
# at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
#
# Parses javadoc-like comments from C++ files and writes them into well-formed
# (hopefully) XML NodeClass definitions, that can then be used by Overflow to
# extract info about the nodes.
#
# Usage:
# Takes a file as input and prints out the XML, so the read the info from
# to_read.cc and output it to nice_xml.def you would need to do something like:
# 'python info2def.py to_read.cc > nice_xml.def.'

from __future__ import print_function

import sys
import re
import fileinput
import copy
import json

defs = json.load(sys.stdin)

print("<?xml version=\"1.0\"?>")
print("<Definitions>")

# read through the file
for node_info in defs['nodes']:

    print("  <NodeClass name=\"{}\" category=\"{}\"".format(node_info['name'], node_info['category']), end="")
    print(" source=\"{}\"".format(node_info['source']), end="")
    if 'require' in node_info :
        print(" require=\"{}\"".format(node_info['require']), end="")
    print(">")

    for _input in node_info['inputs'] :
        print("    <Input name=\"{}\" type=\"{}\">".format(_input['name'], _input['object_type']), end="")
        print(_input['description'], end="")
        print("</Input>")

    for _output in node_info['outputs'] :
        print("    <Output name=\"{}\" type=\"{}\">".format(_output['name'], _output['object_type']), end="")
        print(_output['description'], end="")
        print("</Output>")

    for param in node_info['parameters'] :
        print("    <Parameter name=\"{}\" type=\"{}\" value=\"{}\" list=\"{}\" valid=\"{}\">" \
              .format(param['name'], param['type'], param['value'], \
                      ':'.join(param['list']), param['condition']), end="")
        print(param['description'], end="")
        print("</Parameter>")

    print("    <Description>", end="")
    print(node_info['description'], end="")
    print("</Description>")
    print("  </NodeClass>")

for v in defs['module-depend']:
    module  = v['module']
    depends = v['files']
    print("  <ModuleDepend module=\"{}\">".format(module))
    for depend in depends:
        print("    <Require file=\"{}\"/>".format(depend['file']))
    print("  </ModuleDepend>")

for v in defs['file-depend']:
    file    = v['file']
    depends = v['modules']
    if len(depends) > 0:
        print("  <FileDepend file=\"{}\">".format(file))
        for depend in depends:
            print("    <RequireModule module=\"{}\"/>".format(depend['module']))
        print("  </FileDepend>")

for v in defs['file-depend']:
    file    = v['file']
    depends = v['headers']
    if len(depends) > 0:
        print("  <FileDepend file=\"{}\">".format(file))
        for depend in depends:
            print("    <RequireHeader header=\"{}\"/>".format(depend['header']))
        print("  </FileDepend>")

print("</Definitions>")
